
from os.path import join
import sys
import os
import random
try:
    import configparser
except ImportError:
    # Python 2.x fallback
    import ConfigParser as configparser

#TODO: generalize and pass a dictionary or similar structure to make this flex to changes
def esm_ensemble_generate_configs(exp_id, outpath, start_year, mesh_path):


   fin = open(os.path.join(os.path.dirname(__file__), 'config/fesom', 'namelist.config.tmpl'), "rt")
   # read file contents to string
   data = fin.read()
   # replace all occurrences of the required string

   mapdict = {'{start_year}': start_year, '{mesh_path}': mesh_path}
   data = data.replace('{start_year}', start_year)
   data = data.replace('{mesh_path}', mesh_path)

   file = open(outpath + "/namelist.config", "w")
   file.write(data)
   file.close()



   # close the input file
   #fin.close()
   # open the input file in write mode
   #fin = open("data.txt", "wt")
   # overrite the input file with the resulting data
   #fin.write(data)
   # close the file
   #fin.close()

def esm_ensemble_init(exp_id):

   # 1 - load ensemble config file
   print("Initialization for experiment " + str(exp_id))
   config = configparser.ConfigParser()
   config.read(os.path.join(os.path.dirname(__file__), 'config', 'esm_ensemble.conf'))
   # 2 - create folders for the topdir and each ensemble member runtime folder
   if config.has_option('common', 'top_working_dir'):
      top_dir = config['common']['top_working_dir']
      output_dir = config['common']['output_dir']
      # define the access rights - readable and accessible by all users, and write access by only the owner
      access_rights = 0o755
      try:
         # top dir
         if not os.path.exists(top_dir):
            os.makedirs(top_dir , access_rights)

         # output dir
         if not os.path.exists(output_dir):
            os.makedirs(output_dir, access_rights)

         # get list of members
         sdates_list = (config['common']['ensemble_start_dates']).split()
         print("Creating directories for the following list of dates: " + str(sdates_list))
         for sdate in sdates_list:
            print(sdate)
            if not os.path.exists(top_dir + "/" + sdate):
               os.mkdir(top_dir + "/" + sdate, access_rights)

            if not os.path.exists(output_dir + "/" + sdate):
               os.mkdir(output_dir + "/" + sdate, access_rights)

            #generate the config files for each year/member
            esm_ensemble_generate_configs(exp_id, top_dir + "/" + sdate, sdate, config['fesom2']['mesh_file_path'])

            #create symlink for FESOM2 executable for each working dir
            fesom_exe = config['fesom2']['fesom_binary_path']
            link_name = top_dir + "/" + sdate + "/fesom_x"
            os.remove(link_name)
            os.symlink( fesom_exe , link_name )


      except OSError as exc:  # Python ≥ 2.5
          print("Initialization failed :" + exc.strerror )
          raise
   else:
      print("ERROR - topdir undefined path not found")

# for testing purposes
if __name__ == "__main__":
   print("Running FESOM2 INIT" )
   # first parameter: number of ensemble members
   # 0 - create a ramdon nr for the experiment id
   exp_id = random.randint(100000, 999999)
   esm_ensemble_init(exp_id)
   print("############ All experiment folders created successfully ################")
